@@ -2,29 +2,31 @@
 #  All rights reserved.
 #
 #  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions are met: 
+#  modification, are permitted provided that the following conditions are met:
+#
+#  1. Redistributions of source code must retain the above copyright notice,
+#     this list of conditions and the following disclaimer.
 #
-#  1. Redistributions of source code must retain the above copyright notice, this
-#     list of conditions and the following disclaimer. 
 #  2. Redistributions in binary form must reproduce the above copyright notice,
 #     this list of conditions and the following disclaimer in the documentation
-#     and/or other materials provided with the distribution. 
-#  
-#     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-#     ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-#     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-#     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-#     ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-#     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-#     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-#     ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+#     and/or other materials provided with the distribution.
+#
+#     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+#     IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+#     THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+#     PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+#     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+#     EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+#     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+#     PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+#     LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+#     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 #     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-#  
-#     The views and conclusions contained in the software and documentation are those
-#     of the authors and should not be interpreted as representing official policies, 
-#     either expressed or implied, of the FreeBSD Project.
- 
+#
+#     The views and conclusions contained in the software and documentation are
+#     those of the authors and should not be interpreted as representing
+#     official policies, either expressed or implied, of the FreeBSD Project.
+
 
 import operator
 from functools import wraps
@@ -138,7 +140,7 @@ class ArgumentError(ValueError):
     violates a rule set by the parser. In other words, errors of this type
     should be caught and communicated to the user some how. The default
     behavior is to signal the particular error and show the `usage`.'''
-        
+
     pass
 
 
@@ -154,8 +156,8 @@ class MissingRequiredArgumentError(ArgumentError):
         if isinstance(arg, Option):
             arg = arg.argname
 
-        super(MissingRequiredArgumentError, self).__init__('No value passed for'
-                + ' %s' % arg)
+        super(MissingRequiredArgumentError, self).__init__(
+                'No value passed for %s' % arg)
 
 
 class ManyAllowedNoneSpecifiedArgumentError(ArgumentError):
@@ -192,7 +194,7 @@ class ConflictError(ArgumentError):
     def __init__(self, offender1, offender2):
         super(ConflictError, self).__init__('%s conflicts with %s' %
                 (offender1, offender2))
-        
+
 
 class FailedConditionError(ArgumentError):
     ''' Condition failed. '''
@@ -527,7 +529,10 @@ class Parser(object):
         self._namemaps = {}
         self._rnamemaps = {}
 
-    def help_prefix(self, message):
+    def set_help_prefix(self, message):
+        ''' Indicate text to appear before argument list when the ``help``
+        function is triggered. '''
+
         self._help_prefix = message
         return self
 
@@ -557,6 +562,9 @@ class Parser(object):
         self._double_prefix = flag
         return self
 
+    def use_aliases(self):
+        raise NotImplementedError
+
     @classmethod
     def with_locals(cls):
         ''' Create :class:`Parser` using locals() dict. '''
@@ -584,6 +592,7 @@ class Parser(object):
 
     def float(self, name):
         ''' Add float argument. '''
+
         return self._add_option(name).cast(float)
 
 #    def enum(self, name, values):
@@ -646,12 +655,15 @@ class Parser(object):
 
     def multiword(self, name):
         ''' Accepts multiple terms as an argument. For example:
+
             >>> with Parser() as p:
             ...   p.multiword('multi')
 
             Now accepts:
 
-              python test.py --multi path to something
+            ::
+
+               python test.py --multi path to something
 
         '''
 
@@ -676,6 +688,7 @@ class Parser(object):
 
     def at_least_one(self, *args):
         ''' Require at least one of ``args``. '''
+
         return self._require_at_least_one(*args)
 
     def require_one(self, *args):
@@ -841,12 +854,6 @@ class Parser(object):
             self.print_help()
             sys.exit(0)
 
-    def _check_user_specified(self):
-        for key, values in self._preparsed.iteritems():
-            if len(values) > 1 and key not in self._multiple:
-                raise MultipleSpecifiedArgumentError(('%s specified multiple'
-                        + ' times') % self._to_flag(key))
-
     def _assign(self):
         parsed = {}
         for key, values in self._preparsed.iteritems():
@@ -870,6 +877,13 @@ class Parser(object):
                 raise MissingValueError('%s specified but missing given value'
                         % key)
 
+        # check multiple
+
+        for key, values in self._preparsed.iteritems():
+            if len(values) > 1 and key not in self._multiple:
+                raise MultipleSpecifiedArgumentError(('%s specified multiple'
+                        + ' times') % self._to_flag(key))
+
         # check conditions
         for arg in parsed.iterkeys():
             for cond in self._options[arg]._conditions:
@@ -889,7 +903,8 @@ class Parser(object):
                         break
                 else:
                     if missing:
-                        raise ManyAllowedNoneSpecifiedArgumentError([arg] + missing)
+                        raise ManyAllowedNoneSpecifiedArgumentError([arg] +
+                                missing)
                     else:
                         raise MissingRequiredArgumentError(arg)
 
@@ -935,7 +950,6 @@ class Parser(object):
             self._read(args)
             self._help_if_necessary()
 
-            self._check_user_specified()
             self._assign()
             store = self._store
         except ArgumentError as e:
